/**
 * @author Joel Espinosa Longi
 * @licencia LGPL - http://www.gnu.org/licenses/lgpl.html
 */

var diaEd = (function(diaEd) {

  /**
   * 
   */
  diaEd.Answer = function(content, parentQuestion) {
    this.parentQuestion = parentQuestion;
    this.attributes = new diaEd.Attributes(content, "answer");
    this.questions = [];
  }

  /**
   *
   */
  diaEd.Answer.prototype.buildArrowNextQuestion = function() {
    var question = (this.attributes.get("nextQuestion").trim() != "") ? diaEd.dialog.getQuestionById(this.attributes.get("nextQuestion")) : null;

    diaEd.lineDrawing.addLine(this, question);
  }

  /**
   *
   */
  diaEd.Answer.prototype.render = function(parent) {
    var self = this;

    this.container = $("<li></li>");

    parent.append(this.container);

    // this.container.addClass("ui-state-default");
    this.container.addClass("answerContainer");

    /***********************************************************************************************/
    this.textAnswer = $("<div class='textAnswer'>" + this.attributes.get("description") + "</div>");
    this.textAnswer.disableSelection();
    this.container.append(this.textAnswer);

    /***********************************************************************************************/
    this.removeBtn = $("<div></div>");
    $(this.removeBtn).button( {icons: { primary: "my-icon-delete" }, text: false } ).addClass("removeAnswer");

    /**
     *
     */
    this.removeBtn.mousedown(function(evt) {
      evt.stopPropagation();

      diaEd.elementToDelete = { self: this, question: self.parentQuestion };

      $("#message_deleteAnswer").dialog("open");
    })
    this.removeBtn.disableSelection();
    this.container.append(this.removeBtn);

    /**
     *
     */
    this.container.mousedown(function(evt) {
      $(self.container).addClass("selectedAnswer");

      diaEd.attributesPanel.save(diaEd.currentPanel);

      // unmark the question
      if ((diaEd.currentQuestion != null) && (diaEd.currentQuestion != self)) {
        $(diaEd.currentQuestion.container).removeClass("selectedElement");
      }

      // unmark the answer
      if ((diaEd.currentAnswer != null) && (diaEd.currentAnswer != self)) {
        $(diaEd.currentAnswer.container).removeClass("selectedAnswer");
      }

      diaEd.currentQuestion = self.parentQuestion;
      // diaEd.attributesPanel.save(diaEd.currentPanel);
      diaEd.currentAnswer = self;
      diaEd.attributesPanel.show("#answerAttributes");
    });

  }

  /**
   *
   */
  diaEd.Answer.prototype.fillPanel = function(panelName) {
    // get shortKey
    $(panelName + "_shortKey").val( this.attributes.get("shortKey"));

    // get RADialKey
    $(panelName + "_RADialKey").val( this.attributes.get("RADialKey"));

    // get DescartesKey
    $(panelName + "_DescartesKey").val( this.attributes.get("DescartesKey"));

    // get regexKey
    $(panelName + "_regexKey").val( this.attributes.get("regexKey"));    

    // get bestAnswer
    $(panelName + "_bestAnswer").val( this.attributes.get("bestAnswer"));    

    // get feedback
    $(panelName + "_feedback").val( this.attributes.get("feedback"));

    // get audio
    if (diaEd.enableAudio) {
      $(panelName + "_audio").removeAttr('disabled');
      $(panelName + "_audio").val( this.attributes.get("audio"));
    }
    else {
      $(panelName + "_audio").attr("disabled", "disabled");
      $(panelName + "_audio").val("");
    }

    // get display
    $(panelName + "_display").val( this.attributes.get("display"));    

    // get hide
    $(panelName + "_hide").val( this.attributes.get("hide"));    

    // get set
    $(panelName + "_set").val( this.attributes.get("set"));    

    // get nextQuestion
    $(panelName + "_nextQuestion").val( this.attributes.get("nextQuestion"));    

    // get weight
    $(panelName + "_weight").val( this.attributes.get("weight"));

    // get endDialogue
    $(panelName + "_endDialogue").prop("checked", this.attributes.get("endDialogue"));

    // get description
    $(panelName + "_description").val( this.attributes.get("description"));    
  }

  /**
   *
   */
  diaEd.Answer.prototype.savePanelContent = function(panelName) {
    // get shortKey
    this.attributes.set("shortKey", $(panelName + "_shortKey").val());


    // get RADialKey
    this.attributes.set("RADialKey", $(panelName + "_RADialKey").val());

    // get clave Descartes
    this.attributes.set("DescartesKey", $(panelName + "_DescartesKey").val());

    // get clave RegExp
    this.attributes.set("regexKey", $(panelName + "_regexKey").val());
    
    // get bestAnswer
    this.attributes.set("bestAnswer", $(panelName + "_bestAnswer").val());

    // get feedback
    this.attributes.set("feedback", $(panelName + "_feedback").val());

    // get display
    this.attributes.set("display", $(panelName + "_display").val());

    // get audio
    this.attributes.set("audio", $(panelName + "_audio").val());

    // get hide
    this.attributes.set("hide", $(panelName + "_hide").val());

    // get set
    this.attributes.set("set", $(panelName + "_set").val());

    // get nextQuestion
    this.attributes.set("nextQuestion", $(panelName + "_nextQuestion").val());

    // get weight
    var tmpval = parseInt( $(panelName + "_weight").val() );
    this.attributes.set("weight", isNaN(tmpval)?0:tmpval);

    // set endDialogue
    this.attributes.set("endDialogue", $(panelName + "_endDialogue").prop("checked") );

    // get description
    this.attributes.set("description", $(panelName + "_description").val());
    this.textAnswer.text(this.attributes.get("description"));
  }

  /**
   *
   */
  diaEd.Answer.prototype.toString = function(nivel) {
    var tabularNumber = new Array(nivel+2).join(diaEd.tabularString);

    var str = tabularNumber + "{\n";

    if (this.questions.length === 0) {
      str += this.attributes.toString(nivel+2) + "\n" + tabularNumber + "}";
    }
    else {
      str += this.attributes.toString(nivel+2) + ",";

      for (var i=0, l=this.questions.length; i<l; i++) {
        str += this.questions[i].toString2(nivel) + "\n";
      }

      str += tabularNumber + "}"
    }

    return str;
  }

  return diaEd;
})(diaEd || {});