/**
 * @author Joel Espinosa Longi
 * @licencia LGPL - http://www.gnu.org/licenses/lgpl.html
 */

//decodeURIComponent encodeURIComponent

var diaEd = (function(diaEd) {

  /**
   * 
   */
  diaEd.Dialog = function(content) {
    this.container = $("#contentEditor");
    this.mwContainer = $("#micromundoEditor");

    this.attributes = new diaEd.Attributes(content, "dialog");
    if (content) {
      this.questions = this.getQuestions(content.questions);
      this.micromundos = this.getMicromundos(content.microWorlds);
    }
    else {
      this.questions = this.getQuestions();
      this.micromundos = this.getMicromundos(); 
    }

    // get the reference and remove the children of the container
    this.container.empty();
    this.mwContainer.empty();

    diaEd.currentQuestion = null;

    this.render();
  }

  /**
   *
   */
  diaEd.Dialog.prototype.init = function(dontBuildOptions) {
    this.questionsIDs = [];

    diaEd.lineDrawing.cleanLines();
    
    for (var i=0,l=this.questions.length; i<l; i++) {
      this.questions[i].buildArrows();

      this.questionsIDs.push(this.questions[i].attributes.get("id"));
    }

    if (!dontBuildOptions) {
      this.buildOptionElements();
    }
  }

  /**
   *
   */
   diaEd.Dialog.prototype.buildOptionElements = function() {
    $("#answerAttributes_nextQuestion").empty();
    $("#dialogAttributes_firstQuestion").empty();

    $("#answerAttributes_nextQuestion").append($("<option value=''></option>"));
    $("#dialogAttributes_firstQuestion").append($("<option value=''></option>"));

    for (var i=0, l=this.questionsIDs.length; i<l; i++) {
      $("#answerAttributes_nextQuestion").append($("<option value='" + this.questionsIDs[i] + "'>" + this.questionsIDs[i] + "</option>"))
      $("#dialogAttributes_firstQuestion").append($("<option value='" + this.questionsIDs[i] + "'>" + this.questionsIDs[i] + "</option>"))
    }
   }

  /**
   *
   */
  diaEd.Dialog.prototype.getQuestions = function(childNodes) {
    childNodes = childNodes || [];
    var questions = [];

    for (var i=0, l=childNodes.length; i<l; i++) {
      questions.push( new diaEd.Question(childNodes[i], this.container) );
    }

    if (questions.length === 0) {
      questions.push( new diaEd.Question([], this.container) );
    }

    return questions;
  }

  /**
   *
   */
  diaEd.Dialog.prototype.getMicromundos = function(childNodes) {
    childNodes = childNodes || [];
    var micromundos = [];

    for (var i=0, l=childNodes.length; i<l; i++) {
      micromundos.push( new diaEd.Micromundo(childNodes[i], this.mwContainer) );
    }

    if (micromundos.length === 0) {
      micromundos.push( new diaEd.Micromundo([], this.mwContainer) );
    }

    return micromundos;
  }

  /**
   *
   */
  diaEd.Dialog.prototype.getMicromundosIds = function() {
    var mwIds = [];

    for (var i=0, l=this.micromundos.length; i<l; i++) {
      mwIds.push( this.micromundos[i].attributes.get("id") );
    }

    return mwIds;
  }

  /**
   *
   */
  diaEd.Dialog.prototype.cleanQuestionList = function() {
    var children = $("#contentEditor").children();
    var tmpMw = [];

    for (var i=0, l=children.length; i<l; i++) {
      for (var j=0, k=this.questions.length; j<k; j++) {
        if ($(this.questions[j].container)[0] === children[i]) {
          tmpMw.push(this.questions[j]);
        }
      }      
    }    

    this.questions = tmpMw;
  }


  /**
   *
   */
  diaEd.Dialog.prototype.cleanMicromundoList = function() {
    var children = $("#micromundoEditor").children();
    var tmpMw = [];

    for (var i=0, l=children.length; i<l; i++) {
      for (var j=0, k=this.micromundos.length; j<k; j++) {
        if ($(this.micromundos[j].container)[0] === children[i]) {
          tmpMw.push(this.micromundos[j]);
        }
      }      
    }    

    this.micromundos = tmpMw;
  }  

  /**
   *
   */
  diaEd.Dialog.prototype.addQuestion = function() {
    var children = $(this.container).children();
    var tmpQues = [];

    for (var i=0, l=children.length; i<l; i++) {
      for (var j=0, k=this.questions.length; j<k; j++) {
        if ($(this.questions[j].container)[0] === children[i]) {
          tmpQues.push(this.questions[j]);
        }
      }
    }    

    this.questions = tmpQues;

    this.questions.push( new diaEd.Question({ y: 25 + parseInt( $("#divContainer").scrollTop() ) }, this.container) );
    this.render();
  }

  /**
   *
   */
  diaEd.Dialog.prototype.addMicromundo = function() {
    var children = $(this.mwContainer).children();
    var tmpMw = [];

    for (var i=0, l=children.length; i<l; i++) {
      for (var j=0, k=this.micromundos.length; j<k; j++) {
        if ($(this.micromundos[j].container)[0] === children[i]) {
          tmpMw.push(this.micromundos[j]);
        }
      }
    }

    this.micromundos = tmpMw;

    this.micromundos.push( new diaEd.Micromundo([], this.mwContainer) );
    this.render();
  }

  /**
   *
   */
  diaEd.Dialog.prototype.getQuestionById = function(id) {
    for (var i=0,l=this.questions.length; i<l; i++) {
      if (this.questions[i].attributes.get("id") == id) {
        return this.questions[i];
      }
    }

    return null;
  }

  /**
   * Fill the properties panel with the attributes of the dialog
   */
  diaEd.Dialog.prototype.fillPanel = function(panelName) {
    diaEd.enableAudio = diaEd.dialog.attributes.get("withAudio") && !diaEd.dialog.attributes.get("tts");
    
    // get title
    $(panelName + "_title").val( this.attributes.get("title"));

    // get spellcheck
    if (this.attributes.get("spellcheck").match("auto")) {
      $(panelName + "_spellcheck").prop("checked", true);
    }
    else {
      $(panelName + "_spellcheck").prop("checked", false);
    }

    // get autoresize
    if (this.attributes.get("autoresize")) {
      $(panelName + "_autoresize").prop("checked", true);
    }
    else {
      $(panelName + "_autoresize").prop("checked", false);
    }

    // get delay
    var delay = isNaN(parseFloat(this.attributes.get("delay"))) ? 0 : parseFloat(this.attributes.get("delay"));
    $(panelName + "_delay").val( delay );

    // get withAudio
    var withAudioEnable = this.attributes.get("withAudio");
    if (withAudioEnable) {
      $(panelName + "_withAudio").prop("checked", true);
    }
    else {
      $(panelName + "_withAudio").prop("checked", false);
      $(panelName + "_tts").attr("disabled", "disabled");
    }

    // get tts
    if (this.attributes.get("tts")) {
      $(panelName + "_tts").prop("checked", true);
    }
    else {
      $(panelName + "_tts").prop("checked", false);
    }

    // get withoutServer
    if (this.attributes.get("withoutServer")) {
      $(panelName + "_withoutServer").prop("checked", true);
    }
    else {
      $(panelName + "_withoutServer").prop("checked", false);
    }

    // get styleBalloons
    $(panelName + "_styleBalloons").val( this.attributes.get("styleBalloons"));

    // get firstQuestion
    $(panelName + "_firstQuestion").val( this.attributes.get("firstQuestion"));

  }

  /**
   *
   */
  diaEd.Dialog.prototype.savePanelContent = function(panelName) {
    // set title
    this.attributes.set("title", $(panelName + "_title").val());

    // set spellcheck
    if ($(panelName + "_spellcheck").prop("checked")) {
      this.attributes.set("spellcheck", "auto");
    }
    else {
      this.attributes.set("spellcheck", "off");
    }

    // set autoresize
    this.attributes.set("autoresize", $(panelName + "_autoresize").prop("checked"));

    // set delay
    var delay = isNaN( parseFloat($(panelName + "_delay").val()) ) ? 0 : parseFloat( $(panelName + "_delay").val() );
    $(panelName + "_delay").val( delay );
    this.attributes.set("delay", delay);

    // set withAudio
    this.attributes.set("withAudio", $(panelName + "_withAudio").prop("checked"));

    // set tts
    this.attributes.set("tts", $(panelName + "_tts").prop("checked"));

    // set withoutServer
    this.attributes.set("withoutServer", $(panelName + "_withoutServer").prop("checked"));

    // set styleBalloons
    this.attributes.set("styleBalloons", $(panelName + "_styleBalloons").val());

    // set firstQuestion
    this.attributes.set("firstQuestion", $(panelName + "_firstQuestion").val());    
  }

  /**
   *
   */
  diaEd.Dialog.prototype.render = function() {
    for (var i=0, l=this.questions.length; i<l; i++) {
      this.questions[i].render(this.container);
    }

    for (var i=0, l=this.micromundos.length; i<l; i++) {
      this.micromundos[i].render(this.mwContainer);
    }
  }

  /**
   *
   */
  diaEd.Dialog.prototype.toString = function() {
    var tabularNumber = new Array(2).join(diaEd.tabularString);

    var str = 'var dialogo = \n{\n' + this.attributes.toString(1);

    // has some questions
    if (this.questions.length > 0) {
      str += ',\n' + tabularNumber + '"questions" : [\n';

      for (var i=0,l=this.questions.length; i<l; i++) {
        str += this.questions[i].toString(1) + '\n';
      }

      str = str.substring(0, str.length-2);
      str += '\n' + tabularNumber + ']';
    }

    // has some microworlds
    if (this.micromundos.length > 0) {
      str += ",\n" + tabularNumber + '"microWorlds": [\n'

      for (var i=0,l=this.micromundos.length; i<l; i++) {
        str += this.micromundos[i].toString(2) + "\n";
      }

      str = str.substring(0, str.length-2);
      str += '\n' + tabularNumber + ']';
    }

    return str + "\n}";
  }

  return diaEd;
})(diaEd || {});