/**
 * @author Joel Espinosa Longi
 * @licencia LGPL - http://www.gnu.org/licenses/lgpl.html
 */

var diaEd = (function(diaEd) {

  /**
   * 
   */
  diaEd.Micromundo = function(content, parent) {
    this.parent = parent;

    this.attributes = new diaEd.Attributes(content, "microworld");    
  }

  /**
   *
   */
   diaEd.Micromundo.prototype.render = function(parent) {
    var self = this;

    if (self.container) {
      self.container.remove();
    }

    this.container = $("<div class='microworldContainer'></div>");

    /***********************************************************************************************/
    this.spanText = $("<div style='overflow: hidden; white-space: nowrap;'>" + this.attributes.get("id") + "</div>")
    this.container.append(this.spanText);

    /***********************************************************************************************/
    // this.preview = $("<iframe src='" + this.attributes.get("url") + "'></iframe");
    // this.container.append(this.preview)

    /***********************************************************************************************/
    this.removeBtn = $("<div></div>");
    $(this.removeBtn).button( {icons: { primary: "my-icon-delete" }, text: false } ).addClass("removeMicroworld");

    /**
     *
     */
    this.removeBtn.mousedown(function(evt) {
      evt.stopPropagation();

      diaEd.elementToDelete = { self: this };

      $("#message_deleteMicroworld").dialog("open");
    })
    this.removeBtn.disableSelection();
    this.container.append(this.removeBtn);

    /**
     *
     */
    this.container.mousedown(function(evt) {
      $(self.container).addClass("selectedElement");

      // unmark the microworld
      if ((diaEd.currentMicromundo != null) && (diaEd.currentMicromundo != self)) {
        $(diaEd.currentMicromundo.container).removeClass("selectedElement");
      }

      diaEd.attributesPanel.save(diaEd.currentPanel);
      diaEd.currentMicromundo = self;
      diaEd.attributesPanel.show("#micromundoAttributes");
    });

    /**
     *
     */
    this.container.mouseup(function(evt) {
    });

    parent.append(this.container);
   }

  /**
   * Fill the properties panel with the attributes of the micromundo
   */
  diaEd.Micromundo.prototype.fillPanel = function(panelName) {
    // get identifier
    $(panelName + "_id").val( this.attributes.get("id"));

    // get url
    $(panelName + "_url").val( this.attributes.get("url"));

    // get w
    $(panelName + "_w").val( this.attributes.get("w"));

    // get h
    $(panelName + "_h").val( this.attributes.get("h"));

    // get x
    $(panelName + "_x").val( this.attributes.get("x"));

    // get y
    $(panelName + "_y").val( this.attributes.get("y"));

    // get front
    var withFrontEnable = this.attributes.get("front");
    if (withFrontEnable) {
      $(panelName + "_front").prop("checked", true);
      $(panelName + "_autoDisplay").removeAttr("disabled");
      $(panelName + "_toNextQuestionOnClose").removeAttr("disabled");
    }
    else {
      $(panelName + "_front").prop("checked", false);
      $(panelName + "_autoDisplay").attr("disabled", "disabled");
      $(panelName + "_toNextQuestionOnClose").attr("disabled", "disabled");
    }

    // get autoDisplay
    $(panelName + "_autoDisplay").prop("checked", this.attributes.get("autoDisplay"));

    // get textLink
    $(panelName + "_textLink").val( this.attributes.get("textLink"));

    // get toNextQuestionOnClose
    $(panelName + "_toNextQuestionOnClose").prop("checked", this.attributes.get("toNextQuestionOnClose"));

    // get deactivateChat
    $(panelName + "_deactivateChat").prop("checked", this.attributes.get("deactivateChat"));

    // get reset
    $(panelName + "_reset").prop("checked", this.attributes.get("reset"));
  }

  /**
   *
   */
  diaEd.Micromundo.prototype.savePanelContent = function(panelName) {
    // set identifier
    this.attributes.set("id", $(panelName + "_id").val());
    this.spanText.text(this.attributes.get("id"));

    // set url
    this.attributes.set("url", $(panelName + "_url").val());

    // set w
    if ($(panelName + "_w").val() == "") {
      this.attributes.set("w", null);
    }
    else {
      var tmpval = parseInt( $(panelName + "_w").val() );
      this.attributes.set("w", isNaN(tmpval)?0:tmpval);
    }

    // set h
    if ($(panelName + "_h").val() == "") {
      this.attributes.set("h", null);
    }
    else {
      var tmpval = parseInt( $(panelName + "_h").val() );
      this.attributes.set("h", isNaN(tmpval)?0:tmpval);
    }

    // set x
    if ($(panelName + "_x").val() == "") {
      this.attributes.set("x", null);
    }
    else {
      var tmpval = parseInt( $(panelName + "_x").val() );
      this.attributes.set("x", isNaN(tmpval)?0:tmpval);
    }

    // set y
    if ($(panelName + "_y").val() == "") {
      this.attributes.set("y", null);
    }
    else {
      var tmpval = parseInt( $(panelName + "_y").val() );
      this.attributes.set("y", isNaN(tmpval)?0:tmpval);
    }

    // set front
    this.attributes.set("front", $(panelName + "_front").prop("checked") );

    // set autoDisplay
    this.attributes.set("autoDisplay", $(panelName + "_autoDisplay").prop("checked") );

    // set textLink
    this.attributes.set("textLink", $(panelName + "_textLink").val());

    // set toNextQuestionOnClose
    this.attributes.set("toNextQuestionOnClose", $(panelName + "_toNextQuestionOnClose").prop("checked") );

    // set deactivateChat
    this.attributes.set("deactivateChat", $(panelName + "_deactivateChat").prop("checked") );

    // set reset
    this.attributes.set("reset", $(panelName + "_reset").prop("checked") );
  }

  /**
   *
   */
  diaEd.Micromundo.prototype.toString = function(nivel) {
    return new Array(nivel+1).join(diaEd.tabularString) + "{\n" + this.attributes.toString(nivel+1) + "\n" + new Array(nivel+1).join(diaEd.tabularString) + "},";
  }

  return diaEd;
})(diaEd || {});